
     {*******************************************************************}
     {*                          TWOBRWIN.PAS                           *}
     {*                    Version 1.00 January 1993                    *}
     {*              Copyright (c) 1993 TurboPower Software             *}
     {*                                                                 *}
     {*                         By Lee Inman,                           *}
     {*                      TurboPower Software                        *}
     {*******************************************************************}

{$I BRDEFOPT.INC}

unit BrWinCus;
  {-First of two browser objects for the TwoBrWin demo program }

interface

uses
  WinTypes,
  WinProcs,
  {$IFDEF Ver70}
  Objects,
  OWindows,
  ODialogs,
  {$ELSE}
  WObjects,
  {$ENDIF}
  Strings,

  Filer,
  Rebuild,
  IsamTool,

  LowBrows,      {Low level windows file browser unit}
  WBrowser,      {Windows file browser unit}
  BWCC,          {Borland custom controls}

  BrWinDat,      {Types, constants, and global variables}
  BrWinUtl;      {Utilities and common file functions}

type
  {Window object to contain the customer browser}
  PCustomerWin        = ^TCustomerWin;
  TCustomerWin        = object(TBrowserWindow)
    constructor InitDemo(AParent             : PWindowsObject;
                         ATitle              : PChar;
                         ADrvOrFileBlockPtr  : Pointer;
                         ANumberOfEltsPerRow : Word;
                         ANumberOfRows       : Word;
                         AKeyNr              : Word;
                         ALKey, AHKey        : GenKeyStr);
      {-Initialize the window and connect the browser}
    destructor Done;
      virtual;
      {-Destroy the customer window and associated browser}
    function GetClassName : PChar;
      virtual;
      {-Get the customer window class name}
    procedure SetupWindow;
      virtual;
      {-Basic setup and tweak the system menu}
    procedure WMSysCommand(var Msg: tMessage);
      virtual wm_First + wm_SysCommand;
      {-Intercept the close command and ignore}
    procedure WMGetMinMaxInfo(var Msg: tMessage);
      virtual wm_First+wm_GetMinMaxInfo;
      {-Restrict the minimum size of the window}
    procedure WMMove(var Msg: tMessage);
      virtual wm_First + wm_Move;
      {-Capture move messages}
    procedure GetWindowClass(var WndClass : TWndClass);
      virtual;
      {-Setup the customer window class by adding double clicks}
    procedure wmLButtonDblClk(var Msg : TMessage);
      virtual wm_First + wm_LButtonDblClk;
      {-Invokes the edit dialog for the selected record}
    procedure WMKeyDown(var Msg : TMessage);
      virtual wm_First + wm_KeyDown;
      {-Trap the tab key to allow switching to the order window}
    function BuildRow(var RR : RowRec) : Integer; virtual;
      {-Called by the browser when a row requires construction}
    procedure ShowErrorOccured(Class : Integer);
      virtual;
      {-Called when the browser detects an error}
    procedure DisplayRow(I : Integer; DC : HDC; Inverse : Boolean);
      virtual;
      {-Special hook into the browser to allow detecting when a new
        record is selected and notifying the order window to update}
    function EditDataRec(ATitle : PChar) : Boolean;
      {-Dialog to allow editing/creating/deleting/searching of data records}
    procedure NewRec;
      {-Allows creating new data records}
    procedure DelRec;
      {-Allows deleting the currently selected data record}
    procedure EditRec;
      {-Allosw editing of the currently selected data record}
    procedure Search;
      {-Allows data records to be searched on any field}
    procedure ShowStatus;
      {-Displays a dialog with the current status of the data file}
  end;


  PEditCustomerDialog = ^TEditCustomerDialog;
  TEditCustomerDialog = object(TDialog)
  {-This dialog is used to edit/create/delete/search data records}
    WName             : array[0..20] of Char;  {Window title}
    constructor Init(AParent : PWindowsObject; ATitle : PChar);
    procedure SetupWindow; virtual;
  end;


implementation


  {tCustomerWin}
  constructor tCustomerWin.InitDemo(AParent             : PWindowsObject;
                                    ATitle              : PChar;
                                    ADrvOrFileBlockPtr  : Pointer;
                                    ANumberOfEltsPerRow : Word;
                                    ANumberOfRows       : Word;
                                    AKeyNr              : Word;
                                    ALKey, AHKey        : GenKeyStr);
  var
    R                   : tRect;

  begin
    if not TBrowserWindow.Init(AParent, ATitle) then
      Fail;

    {Build the header line for the customer window}
    CustomerHdr :=
      Extend(
        Extend('Cust#', 5) + ' ' +
        Extend('Name: Last, First', 30) + ' ' +
        Extend('Address', 25) + ' ' +
        Extend('City, State Zip', 32), MaxCols);

    {Attach a browser from the LowBrows unit}
    if not ConnectLowBrowser(
      New(PLowWinBrowser,
          Init(True, ADrvOrFileBlockPtr, ANumberOfEltsPerRow, ANumberOfRows,
               AKeyNr, ALKey, AHKey, Customer, False)), CustomerHdr, '') then
      Fail;

    {Set a timer for refreshing the screen in multi-user situations}
    if BTNetSupported <> NoNet then
      SetTimer(hWindow, 1, 10000, nil);
  end;

  destructor tCustomerWin.Done;
  begin
    {Destroy the timer if one was used}
    if BTNetSupported <> NoNet then
      KillTimer(hWindow, 1);
    TBrowserWindow.Done;
  end;

  function tCustomerWin.GetClassName : PChar;
  begin
    GetClassName := 'CustomerWin';
  end;

  procedure tCustomerWin.GetWindowClass(var WndClass : TWndClass);
  begin
    TWindow.GetWindowClass(WndClass);
    {Set flag so we can edit when double clicked}
    WndClass.Style := WndClass.Style or cs_DblClks;
  end;

  procedure tCustomerWin.SetupWindow;
  var
    M                   : hMenu;
  begin
    TBrowserWindow.SetupWindow;
    {Remove close from the system menu - closing the window would destroy
     the link between the two file blocks and crash this program}
    M := GetSystemMenu(hWindow, False);
    DeleteMenu(M, 6, mf_ByPosition); {The Close item}
    DeleteMenu(M, 6, mf_ByPosition); {The separator}
  end;

  procedure tCustomerWin.WMSysCommand(var Msg: tMessage);
  begin
    if Msg.wParam <> sc_Close then
      DefWndProc(Msg);  {Else, ignore}
  end;

  procedure tCustomerWin.WMGetMinMaxInfo;
  type
    pMinMaxInfo      = ^tMinMaxInfo;
    tMinMaxInfo      = record
      ptReserved     : tPoint;
      ptMaxSize      : tPoint;
      ptMaxPosition  : tPoint;
      ptMinTrackSize : tPoint;
      ptMaxTrackSize : tPoint;
    end;

  var
    UsedDC           : hDC;
    TM               : tTextMetric;
    H,W              : Integer;

  begin
    DefWndProc(Msg);
    UsedDC := GetDC(hWindow);
    if GetTextMetrics(UsedDC,TM) then begin
      with TM do begin
        H := tmHeight + tmExternalLeading;
        W := tmMaxCharWidth;
      end;
      with Msg do begin
        pMinMaxInfo(lParam)^.ptMinTrackSize.x := 20 * W;
        pMinMaxInfo(lParam)^.ptMinTrackSize.y := 3 * H +
                                                 GetSystemMetrics(sm_CYCaption) +
                                                 GetSystemMetrics(sm_CYHScroll) +
                                                 GetSystemMetrics(sm_CYDlgFrame);
      end;
    end;
    ReleaseDC(hWindow,UsedDC);
    Msg.Result := 0; {Msg processed}
  end;

  procedure tCustomerWin.WMMove(var Msg: tMessage);
  var
    R                : tRect;
    UsedDC           : hDC;
    TM               : tTextMetric;
    H                : Integer;

  begin
    DefWndProc(Msg);

    {A small area below the last line displayed is not painted in some situations.}
    {i.e. if both windows are moved below the bottom of the main window, worked with}
    { and then tiled or moved so that the bottom of the window is visable. To see why}
    { this happens, look at the Paint and DisplayRow methods in WBrowser.}

    {Force painting of the bottom of the window}

    {Get info about current text size}
    UsedDC := GetDC(hWindow);
    if not GetTextMetrics(UsedDC,TM) then Exit;
    H := TM.tmHeight + TM.tmExternalLeading;
    ReleaseDC(hWindow,UsedDC);

    GetClientRect(hWindow,R);
    R.Top := R.Bottom - H;

    {Erase an area of one line hieght at the bottom of the window}
    InvalidateRect(hWindow,@R,True);
    UpdateWindow(hWindow);
  end;

  function tCustomerWin.BuildRow(var RR : RowRec) : Integer;
  {-Construct a display string for this record}
  begin
    BuildRow := NoError;  {Assume success}
    with Customer.Data do
      if RR.Status <> NoError then
        RR.Row := Extend('****  ' + RR.IKS, MaxCols)
      else
        RR.Row := Extend(
          Extend(StrPas(CustNum), 5) + ' ' +
          Extend(StrPas(LastName) + ', ' +
                 StrPas(FirstName), 30) + ' ' +
          Extend(StrPas(Address), 25) + ' ' +
          Extend(StrPas(City) + ', ' +
                 StrPas(State) + ' ' +
                 StrPas(Zip), 32), MaxCols);
  end;

  procedure tCustomerWin.ShowErrorOccured(Class : Integer);
  begin
    if Class > 1 then
      if YesNo('Abort program') then
        Halt
      else
        {suspend timer events until a next action is done}
        SetSuppressTimer(True);
  end;

  procedure tCustomerWin.DisplayRow(I : Integer; DC : HDC; Inverse : Boolean);
  {-Special hook into the browser to allow detecting when a new record is
    selected and notifying the order window to update -- This method is
    originally defined in MedBrows unit and is called for each record that
    is to be displayed in the current window.}
  var
    Match : Boolean;
  begin
    if Inverse then  {If this is the currently selected item}
      if GetCurrentRec(Match) = 0 then begin
        CurrentCustomer := Customer.Data;
        {Send a message to parent to update the orders window}
        PostMessage(Parent^.hWindow, wm_Command, cm_UpdateOrders, 0);
      end;
    TBrowserWindow.DisplayRow(I, DC, Inverse);
  end;

  procedure tCustomerWin.wmLButtonDblClk(var Msg : TMessage);
  begin
    {Send message to parent about a request to edit this record}
    PostMessage(Parent^.hWindow, wm_Command, cm_DblClkCust, 0);
  end;

  procedure tCustomerWin.WMKeyDown(var Msg : TMessage);
  begin
    {Look for the tab key to allow switching to the other window}
    if Msg.wParam = vk_Tab then
      {Notify parent to change focus to the order window}
      PostMessage(Parent^.hWindow, wm_Command, cm_WindowTab, 0);
    TBrowserWindow.WMKeyDown(Msg);
  end;

  function tCustomerWin.EditDataRec(ATitle : PChar) : Boolean;
  var
    Dialog              : PEditCustomerDialog;
  begin
    Dialog := New(PEditCustomerDialog, Init(@Self, ATitle));
    EditDataRec := Application^.ExecDialog(Dialog) = id_OK;
  end;

  procedure tCustomerWin.EditRec;
  var
    SaveCustomer        : CustomerDef;
    Match               : Boolean;
  begin
    if GetCurrentRec(Match) > 0 then
      Exit;
    SaveCustomer := Customer;
    if EditDataRec('Edit') then
      if not ChangeCustomerRecAndKeys(SaveCustomer, GetCurrentDatRef) then
        DbError('Error updating record')
      else
        SetAndUpdateBrowserScreen(BuildCustomerKey(Customer, 1), GetCurrentDatRef);
  end;

  procedure tCustomerWin.NewRec;
  var
    Key                 : IsamKeyStr;
    Ref                 : LongInt;
  begin
    FillChar(Customer, SizeOf(Customer), 0);
    if EditDataRec('New') then
      if not AddCustomerRecAndKeys(Key, Ref) then
        DbError('Error adding record')
      else
        SetAndUpdateBrowserScreen(Key, Ref);
  end;

  procedure tCustomerWin.DelRec;
  var
    SaveCustomer        : CustomerDef;
    Key,KeyHold         : IsamKeyStr;
    Ref                 : LongInt;
    Match               : Boolean;
  begin
    if GetCurrentRec(Match) > 0 then
      Exit;
    {See if this customer has any related orders in the order file}
    Key := BuildCustomerKey(Customer, 1);
    KeyHold := Key;
    BTSearchKey(OrderFB, 1, Ref, Key);
    if (Key = KeyHold) and (IsamError <> 10210) then begin
      MessageBox(GetFocus,
      'This customer has order entries.  Delete orders first.',
      'Delete Error',mb_IconStop+mb_Ok);
      Exit;
    end;



    SaveCustomer := Customer;
    if EditDataRec('Delete') then begin
      Customer := SaveCustomer;
      if not DeleteCustomerRecAndKeys(GetCurrentDatRef) then
        DbError('Error deleting record')
      else
        UpdateBrowserScreen;
    end;
  end;

  procedure tCustomerWin.Search;
  var
    Key                 : IsamKeyStr;
    Ref                 : LongInt;
  begin
    FillChar(Customer, SizeOf(Customer), 0);
    if EditDataRec('Search') then begin
      Key := BuildCustomerKey(Customer, 1);
      if not SearchCustomerForKey(Key, Ref) then
        DbError('Search error')
      else
        SetAndUpdateBrowserScreen(Key, Ref);
    end;
  end;

  procedure tCustomerWin.ShowStatus;
  var
    Result              : Integer;
    UK                  : LongInt;
    S, T                : String;
    Buffer              : array[0..255] of Char;
  begin
    SetSuppressTimer(True);
    UK := BTUsedKeys(CustomerFB, GetCurrentKeyNr);
    Result := BTIsamErrorClass;
    if Result = NoError then begin
      Str(UK:0, S);
      Str(BTGetInternalDialogID(CustomerFB):0, T);
      S := 'Records : ' + S + '    Task : ' + T;
      StrPCopy(Buffer, S);
      BwccMessageBox(GetActiveWindow, @Buffer, 'Status', mb_OK or mb_TaskModal);
    end;
    SetSuppressTimer(False);
  end;


  {tEditCustomerDialog}
  constructor TEditCustomerDialog.Init(AParent             : PWindowsObject;
                                       ATitle              : PChar);
  var
    CP                  : PControl;
  begin
    TDialog.Init(AParent, 'CUSTOMER');
    StrCopy(WName, ATitle);
    TransferBuffer := @Customer.Data;
    EnableTransfer;
    with Customer.Data do begin
      CP := New(PEdit, InitResource(@Self, id_CustNum, SizeOf(CustNum)));
      CP := New(PEdit, InitResource(@Self, id_Last, SizeOf(LastName)));
      CP := New(PEdit, InitResource(@Self, id_First, SizeOf(FirstName)));
      CP := New(PEdit, InitResource(@Self, id_Address, SizeOf(Address)));
      CP := New(PEdit, InitResource(@Self, id_City, SizeOf(City)));
      CP := New(PEdit, InitResource(@Self, id_State, SizeOf(State)));
      CP := New(PEdit, InitResource(@Self, id_ZipCode, SizeOf(Zip)));
      CP := New(PEdit, InitResource(@Self, id_Phone, SizeOf(Phone)));
    end;
  end;

  procedure TEditCustomerDialog.SetupWindow;
  begin
    TDialog.SetupWindow;
    SetWindowText(hWindow, WName);
  end;


end.

