
     {*******************************************************************}
     {*                          TWOBRWIN.PAS                           *}
     {*                    Version 1.00 January 1993                    *}
     {*              Copyright (c) 1993 TurboPower Software             *}
     {*                                                                 *}
     {*                         By Lee Inman,                           *}
     {*                      TurboPower Software                        *}
     {*******************************************************************}

{$I BRDEFOPT.INC}

unit BrWinOrd;
  {-Second of two browser objects for the TwoBrWin demo program}

interface

uses
  WinTypes,
  WinProcs,
  {$IFDEF Ver70}
  Objects,
  OWindows,
  ODialogs,
  {$ELSE}
  WObjects,
  {$ENDIF}
  Strings,
  Filer,
  Rebuild,
  IsamTool,
  LowBrows,        {Low level windows file browser unit}
  WBrowser,        {Windows file browser unit}
  BWCC,            {Borland custom controls}

  BrWinDat,        {Types, constants, and global variables}
  BrWinUtl;        {Utilities and common file functions}


type

  POrderWin           = ^TOrderWin;
  TOrderWin           = object(TBrowserWindow)
    FilterOn            : Boolean;  {Flag for record filtering}
    constructor InitDemo(AParent             : PWindowsObject;
                         ATitle              : PChar;
                         ADrvOrFileBlockPtr  : Pointer;
                         ANumberOfEltsPerRow : Word;
                         ANumberOfRows       : Word;
                         AKeyNr              : Word;
                         ALKey, AHKey        : GenKeyStr);
    destructor Done;
      virtual;
    procedure SetupWindow;
      virtual;
      {-Basic setup and tweak the system menu}
    procedure WMSysCommand(var Msg: tMessage);
      virtual wm_First + wm_SysCommand;
      {-Intercept the close command and ignore}
    procedure WMGetMinMaxInfo(var Msg: tMessage);
      virtual wm_First+wm_GetMinMaxInfo;
      {-Restrict the minimum size of the window}
    procedure WMMove(var Msg: tMessage);
      virtual wm_First + wm_Move;
      {-Capture move messages}
    function PreCompletePage : Integer;
      virtual;
    function PostCompletePage : Integer;
      virtual;
    function GetClassName : PChar;
      virtual;
    procedure GetWindowClass(var WndClass : TWndClass);
      virtual;
    procedure wmLButtonDblClk(var Msg : TMessage);
      virtual wm_First + wm_LButtonDblClk;
    procedure WMKeyDown(var Msg : TMessage);
      virtual wm_First + wm_KeyDown;
    function BuildRow(var RR : RowRec) : Integer;
      virtual;
    function PerformFilter(var RR              : RowRec;
                           var UseIt           : Boolean) : Integer;
      virtual;
    procedure ShowErrorOccured(Class : Integer);
      virtual;

    {Added this procedure to allow changing key ranges implemented in}
    {the BRMBrowser object}
    procedure ChangeKeyRange(ALKey, AHKey : GenKeyStr);

    function EditDataRec(ATitle : PChar) : Boolean;
    procedure NewRec(CustNum : PChar);
    procedure DelRec;
    procedure EditRec;
    procedure FilterOnOff;
    procedure ShowStatus;
  end;

  PEditOrderDialog    = ^TEditOrderDialog;
  TEditOrderDialog    = object(TDialog)
  {-This dialog is used to edit/create/delete/search data records}
    WName             : array[0..20] of Char;  {Window Title}
    constructor Init(AParent : PWindowsObject; ATitle : PChar);
    procedure SetupWindow; virtual;
  end;


implementation


  {tOrderWin}
  constructor tOrderWin.InitDemo(AParent             : PWindowsObject;
                                 ATitle              : PChar;
                                 ADrvOrFileBlockPtr  : Pointer;
                                 ANumberOfEltsPerRow : Word;
                                 ANumberOfRows       : Word;
                                 AKeyNr              : Word;
                                 ALKey, AHKey        : GenKeyStr);
  var
    R                   : tRect;

  begin
    if not TBrowserWindow.Init(AParent, ATitle) then
      Fail;

    FilterOn := False;  {No record filtering to start with}

    {Build the header line for the order window}
    OrderHdr :=
      Extend(
        Extend('Product', 25) + ' ' +
        Extend('Price', 15) + ' ' +
        Extend('Balance', 15), MaxCols);

    {Attach a browser from the LowBrows unit}
    if not ConnectLowBrowser(
      New(PLowWinBrowser,
          Init(True, ADrvOrFileBlockPtr, ANumberOfEltsPerRow, ANumberOfRows,
               AKeyNr, ALKey, AHKey, Order, False)), OrderHdr, '') then
      Fail;

    {Set a timer for refreshing screen in multi-user situations}
    if BTNetSupported <> NoNet then
      SetTimer(hWindow, 1, 10000, nil);
  end;

  destructor tOrderWin.Done;
  begin
    {Destroy the timer if one was used}
    if BTNetSupported <> NoNet then
      KillTimer(hWindow, 1);
    TBrowserWindow.Done;
  end;

  function tOrderWin.GetClassName : PChar;
  begin
    GetClassName := 'OrderWin';
  end;

  procedure tOrderWin.GetWindowClass(var WndClass : TWndClass);
  begin
    TWindow.GetWindowClass(WndClass);
    {Set flag so we can edit when double clicked}
    WndClass.Style := WndClass.Style or cs_DblClks;
  end;

  procedure tOrderWin.SetupWindow;
  var
    M                   : hMenu;
  begin
    TBrowserWindow.SetupWindow;

    {Remove close from the system menu - closing the window would destroy
     the link between the two file blocks and crash this program}
    M := GetSystemMenu(hWindow, False);
    DeleteMenu(M, 6, mf_ByPosition); {The Close item}
    DeleteMenu(M, 6, mf_ByPosition); {The separator}
  end;

  procedure tOrderWin.WMSysCommand(var Msg: tMessage);
  begin
    if Msg.wParam <> sc_Close then
      DefWndProc(Msg);  {Else, ignore}
  end;

  procedure tOrderWin.WMGetMinMaxInfo;
  type
    pMinMaxInfo      = ^tMinMaxInfo;
    tMinMaxInfo      = record
      ptReserved     : tPoint;
      ptMaxSize      : tPoint;
      ptMaxPosition  : tPoint;
      ptMinTrackSize : tPoint;
      ptMaxTrackSize : tPoint;
    end;

  var
    UsedDC           : hDC;
    TM               : tTextMetric;
    H,W              : Integer;

  begin
    DefWndProc(Msg);
    UsedDC := GetDC(hWindow);
    if GetTextMetrics(UsedDC,TM) then begin
      with TM do begin
        H := tmHeight + tmExternalLeading;
        W := tmMaxCharWidth;
      end;
      with Msg do begin
        pMinMaxInfo(lParam)^.ptMinTrackSize.x := 20 * W;
        pMinMaxInfo(lParam)^.ptMinTrackSize.y := 3 * H +
                                                 GetSystemMetrics(sm_CYCaption) +
                                                 GetSystemMetrics(sm_CYHScroll) +
                                                 GetSystemMetrics(sm_CYDlgFrame);
      end;
    end;
    ReleaseDC(hWindow,UsedDC);
    Msg.Result := 0; {Msg processed}
  end;

  procedure tOrderWin.WMMove(var Msg: tMessage);
  var
    R                : tRect;
    UsedDC           : hDC;
    TM               : tTextMetric;
    H                : Integer;

  begin
    DefWndProc(Msg);

    {A small area below the last line displayed is not painted in some situations.}
    {i.e. if both windows are moved below the bottom of the main window, worked with}
    { and then tiled or moved so that the bottom of the window is visable. To see why}
    { this happens, look at the Paint and DisplayRow methods in WBrowser.}

    {Force painting of the bottom of the window}

    {Get info about current text size}
    UsedDC := GetDC(hWindow);
    if not GetTextMetrics(UsedDC,TM) then Exit;
    H := TM.tmHeight + TM.tmExternalLeading;
    ReleaseDC(hWindow,UsedDC);

    GetClientRect(hWindow,R);
    R.Top := R.Bottom - H;

    {Erase an area of one line hieght at the bottom of the window}
    InvalidateRect(hWindow,@R,True);
    UpdateWindow(hWindow);
  end;

  function tOrderWin.PreCompletePage : Integer;
  begin
    PreCompletePage := 0;
  end;

  function tOrderWin.PostCompletePage : Integer;
  begin
    PostCompletePage := 0;
  end;

  function tOrderWin.BuildRow(var RR : RowRec) : Integer;
  {-Construct a display string for this record}
  begin
    BuildRow := NoError;  {Assume success}
    with Order.Data do
      if RR.Status <> NoError then
        RR.Row := Extend('****  ' + RR.IKS, MaxCols)
      else
        RR.Row := Extend(
          Extend(StrPas(Product), 25) + ' ' +
          Extend(StrPas(Amount), 15) + ' ' +
          Extend(StrPas(Balance), 15), MaxCols);
  end;

  function tOrderWin.PerformFilter(var RR : RowRec; var UseIt : Boolean) : Integer;
  var
    R                   : Real;
    I                   : Integer;

  begin
    PerformFilter := NoError;
    UseIt := True;
    if not FilterOn then Exit;
    {if filter is on - only show records with open balances }
    if FilterOn then begin
      UseIt := False;
      I := GetThisRec(RR);
      if I > 0 then begin
        PerformFilter := I;  {Return error code}
        Exit;
      end;
      Val(Order.Data.Balance, R, I);
      {Show only orders with balances less than $1000.00}
      if (I = 0) and (R < 1000.00) then begin
        UseIt := True;
        PerformFilter := BuildBrowScreenRow(RR);  {Return any error code}
      end;
    end;
  end;

  procedure tOrderWin.ShowErrorOccured(Class : Integer);
  begin
    if Class > 1 then
      if YesNo('Abort program') then
        Halt
      else
        {suspend timer events until a next action is done}
        SetSuppressTimer(True);
  end;

  procedure tOrderWin.wmLButtonDblClk(var Msg : TMessage);
  begin
    {Send message to parent about a request to edit this record}
    PostMessage(Parent^.hWindow, wm_Command, cm_DblClkOrder, 0);
  end;

  procedure tOrderWin.WMKeyDown(var Msg : TMessage);
  begin
    {Look for the tab key to allow switching to the other window}
    if Msg.wParam = vk_Tab then
      PostMessage(Parent^.hWindow, wm_Command, cm_WindowTab, 0);
    TBrowserWindow.WMKeyDown(Msg);
  end;

  procedure tOrderWin.ChangeKeyRange(ALKey, AHKey : GenKeyStr);
  begin
    {BRMBrowser looks at the high and low keys to decide what range
     of records to display -- this is why this window shows only
     the items associated with the currently selected customer. This
     is much faster than using the filter mechanism.}
    BrowserPtr^.LowKey := ALKey;
    BrowserPtr^.HighKey := AHKey;
  end;

  function tOrderWin.EditDataRec(ATitle : PChar) : Boolean;
  var
    Dialog              : PEditOrderDialog;
  begin
    Dialog := New(PEditOrderDialog, Init(@Self, ATitle));
    EditDataRec := Application^.ExecDialog(Dialog) = id_OK;
  end;

  procedure tOrderWin.EditRec;
  var
    SaveOrder           : OrderDef;
    Match               :  Boolean;
  begin
    if GetCurrentRec(Match) > 0 then
      Exit;
    SaveOrder := Order;
    if EditDataRec('Edit') then
      if not ChangeOrderRecAndKeys(SaveOrder, GetCurrentDatRef) then
        DbError('Error updating record')
      else
        SetAndUpdateBrowserScreen(BuildOrderKey(Order, 1), GetCurrentDatRef);
  end;

  procedure tOrderWin.NewRec(CustNum : PChar);
  var
    Key                 : IsamKeyStr;
    Ref                 : LongInt;
  begin
    FillChar(Order, SizeOf(Order), 0);
    if CustNum <> nil then
      StrCopy(Order.Data.CustNum, CustNum);
    if EditDataRec('New') then
      if not AddOrderRecAndKeys(Key, Ref) then
        DbError('Error adding record')
      else
        SetAndUpdateBrowserScreen(Key, Ref);
  end;

  procedure tOrderWin.DelRec;
  var
    SaveOrder           : OrderDef;
    Match               : Boolean;
  begin
    if GetCurrentRec(Match) > 0 then
      Exit;
    SaveOrder := Order;
    if EditDataRec('Delete') then begin
      Order := SaveOrder;
      if not DeleteOrderRecAndKeys(GetCurrentDatRef) then
        DbError('Error deleting record')
      else
        UpdateBrowserScreen;
    end;
  end;

  procedure tOrderWin.FilterOnOff;
  begin
    {Use the current settings for filtering the records}
    FilterOn := not FilterOn;
    UpdateBrowserScreen;
  end;

  procedure tOrderWin.ShowStatus;
  var
    Result              : Integer;
    UK                  : LongInt;
    S, T                : String;
    Buffer              : array[0..255] of Char;
  begin
    SetSuppressTimer(True);
    UK := BTUsedKeys(OrderFB, GetCurrentKeyNr);
    Result := BTIsamErrorClass;
    if Result = NoError then begin
      Str(UK:0, S);
      Str(BTGetInternalDialogID(OrderFB):0, T);
      S := 'Records : ' + S + '    Task : ' + T;
      StrPCopy(Buffer, S);
      BwccMessageBox(GetActiveWindow, @Buffer, 'Status', mb_OK or mb_TaskModal);
    end;
    SetSuppressTimer(False);
  end;


  {tEditOrderDialog}
  constructor TEditOrderDialog.Init(AParent             : PWindowsObject;
                                    ATitle              : PChar);
  var
    CP                  : PControl;
  begin
    TDialog.Init(AParent, 'ORDER');
    StrCopy(WName, ATitle);
    TransferBuffer := @Order.Data;
    EnableTransfer;
    with Order.Data do begin
      CP := New(PEdit, InitResource(@Self, id_CustNum, SizeOf(CustNum)));
      CP := New(PEdit, InitResource(@Self, id_Product, SizeOf(Product)));
      CP := New(PEdit, InitResource(@Self, id_Amount, SizeOf(Amount)));
      CP := New(PEdit, InitResource(@Self, id_Balance, SizeOf(Balance)));
    end;
  end;

  procedure TEditOrderDialog.SetupWindow;
  begin
    TDialog.SetupWindow;
    SetWindowText(hWindow, WName);
  end;

end.

