
     {*******************************************************************}
     {*                          TWOBRWIN.PAS                           *}
     {*                    Version 1.00 January 1993                    *}
     {*              Copyright (c) 1993 TurboPower Software             *}
     {*                                                                 *}
     {*                         By Lee Inman,                           *}
     {*                      TurboPower Software                        *}
     {*******************************************************************}

{$I BRDEFOPT.INC}

unit BrWinUtl;
  {-File and utility functions for the TwoBrWin demo program}

interface

uses
  WinTypes,
  WinProcs,
  {$IFDEF Ver70}
  Objects,
  OWindows,
  ODialogs,
  {$ELSE}
  WObjects,
  {$ENDIF}
  Strings,
  Filer,
  Rebuild,
  IsamTool,
  LowBrows,
  WBrowser,
  BWCC,

  BrWinDat;


  {General utility functions}
  procedure DbError(S : PChar);
  procedure AutoErrorDisplay;
  function YesNo(S : String) : Boolean;
  function Extend(S : String; Nr : Byte) : String;
  procedure ShowMessage(S, Title : PChar);
  function IsLockError : Boolean;
  function LockAbort  : Boolean;

  {Customer related functions}
  function OpenCustomerFile : Boolean;
  function CreateCustomerFile : Boolean;
  function RebuildCustomerFile : Boolean;
  function AddCustomerData(P : CustomerDef; var Ref : LongInt) : Boolean;
  function ChangeCustomerData(P : CustomerDef; Ref : LongInt) : Boolean;
  function DeleteCustomerData(Ref : LongInt) : Boolean;
  function AddACustomerKey(K : IsamKeyStr; Ref : LongInt; KeyNr : Integer) : Boolean;
  function DeleteACustomerKey(K : IsamKeyStr; Ref : LongInt; KeyNr : Integer) : Boolean;
  function ChangeACustomerKey(OldK, NewK : IsamKeyStr; Ref : LongInt;
                              KeyNr               : Integer) : Boolean;
  function AddCustomerRecAndKeys(var Key : IsamKeyStr; var Ref : LongInt) : Boolean;
  function ChangeCustomerRecAndKeys(var OldCustomer     : CustomerDef;
                                    Ref                 : LongInt) : Boolean;
  function DeleteCustomerRecAndKeys(Ref : LongInt) : Boolean;
  function SearchCustomerForKey(var Key : IsamKeyStr; var DatRef : LongInt) : Boolean;
  function BuildCustomerKey(var P; KeyNr : Word) : IsamKeyStr;

  {Order related functions}
  function OpenOrderFile : Boolean;
  function CreateOrderFile : Boolean;
  function RebuildOrderFile : Boolean;
  function AddOrderData(P : OrderDef; var Ref : LongInt) : Boolean;
  function ChangeOrderData(P : OrderDef; Ref : LongInt) : Boolean;
  function DeleteOrderData(Ref : LongInt) : Boolean;
  function AddAOrderKey(K : IsamKeyStr; Ref : LongInt; KeyNr : Integer) : Boolean;
  function DeleteAOrderKey(K : IsamKeyStr; Ref : LongInt; KeyNr : Integer) : Boolean;
  function ChangeAOrderKey(OldK, NewK : IsamKeyStr; Ref : LongInt;
                           KeyNr               : Integer) : Boolean;
  function AddOrderRecAndKeys(var Key : IsamKeyStr; var Ref : LongInt) : Boolean;
  function ChangeOrderRecAndKeys(var OldOrder        : OrderDef;
                                 Ref                 : LongInt) : Boolean;
  function DeleteOrderRecAndKeys(Ref : LongInt) : Boolean;
  function SearchOrderForKey(var Key : IsamKeyStr; var DatRef : LongInt) : Boolean;
  function BuildOrderKey(var P; KeyNr : Word) : IsamKeyStr;


implementation

  {Utility functions}

  procedure DbError(S : PChar);
  begin
    BwccMessageBox(
      GetActiveWindow, S, 'Data Base Error', mb_OK or mb_TaskModal);
  end;

  procedure AutoErrorDisplay;
  var
    CharAr              : array[0..255] of Char;
  begin
    if IsamError <> 0 then begin
      StrPCopy(CharAr, IsamErrorMessage(IsamError));
      DbError(CharAr);
    end;
  end;

  function YesNo(S : String) : Boolean;
  var
    SZ                  : array[0..255] of Char;
  begin
    StrPCopy(SZ, S + '?');
    YesNo := BwccMessageBox(
      GetActiveWindow, SZ, 'User Question', mb_YesNo or mb_IconHand) = id_Yes;
  end;

  function Extend(S : String; Nr : Byte) : String;
  var
    S1                  : String;
  begin
    S1 := S;
    while Length(S1) < Nr do
      S1 := S1 + ' ';
    Extend := S1;
  end;

  procedure ShowMessage(S, Title : PChar);
  begin
    BwccMessageBox(GetActiveWindow, S, Title, mb_OK or mb_TaskModal);
  end;

  function IsLockError : Boolean;
  begin
    IsLockError := BTIsamErrorClass = 2;
  end;

  function LockAbort  : Boolean;
  begin
    LockEnd := IsLockError;
    if not LockEnd then
      LockAbort := False
    else
      LockAbort := YesNo('A lock prevents this operation. Abort');
  end;


  {Customer functions}

  function OpenCustomerFile : Boolean;
  var
    OK, OK1             : Boolean;
  begin
    OpenCustomerFile := False;
    OK := False;
    repeat
      BTOpenFileBlock(
        CustomerFB, CustomerFBName, False, False, False, True);
      OK := IsamOK;
      if not OK then begin
        OK1 := False;
        if IsamError = 10010 then begin
          if YesNo('Index defective. Rebuild') then
            OK1 := RebuildCustomerFile;
        end
        else if IsamError = 9903 then begin
          if YesNo('Customer data file does not exist. Create') then begin
            OK1 := CreateCustomerFile;
            if not OK1 then
              Exit;
          end
          else
            Exit;
        end
        else
          Exit;
      end;
    until OK;
    OpenCustomerFile := True;
  end;

  function CreateCustomerFile : Boolean;
  var
    IID                 : IsamIndDescr;
  begin
    IID[1].Keyl := 5;
    IID[1].AllowDupK := False;
    BTCreateFileBlock(CustomerFBName, SizeOf(CustomerDef), 1, IID);
    CreateCustomerFile := IsamOK;
  end;

  function RebuildCustomerFile : Boolean;
  var
    IID                 : IsamIndDescr;
  begin
    IID[1].Keyl := 5;
    IID[1].AllowDupK := False;
    RebuildFileBlock(
      CustomerFBName, SizeOf(CustomerDef), 1, IID, @BuildCustomerKey);
    RebuildCustomerFile := IsamOK;
  end;

  function AddCustomerData(P : CustomerDef; var Ref : LongInt) : Boolean;
  begin
    AddCustomerData := False;
    repeat
      BTAddRec(CustomerFB, Ref, P);
      if LockAbort then
        Exit;
    until not LockEnd;
    AddCustomerData := IsamOK;
  end;

  function ChangeCustomerData(P : CustomerDef; Ref : LongInt) : Boolean;
  begin
    ChangeCustomerData := False;
    repeat
      BTPutRec(CustomerFB, Ref, P, False);
      if LockAbort then
        Exit;
    until not LockEnd;
    ChangeCustomerData := IsamOK;
  end;

  function DeleteCustomerData(Ref : LongInt) : Boolean;
  begin
    DeleteCustomerData := False;
    repeat
      BTDeleteRec(CustomerFB, Ref);
      if LockAbort then
        Exit;
    until not LockEnd;
    DeleteCustomerData := IsamOK;
  end;

  function AddACustomerKey(K : IsamKeyStr; Ref : LongInt; KeyNr : Integer) : Boolean;
  begin
    AddACustomerKey := False;
    repeat
      BTAddKey(CustomerFB, KeyNr, Ref, K);
      if LockAbort then
        Exit;
    until not LockEnd;
    AddACustomerKey := IsamOK;
  end;

  function DeleteACustomerKey(K : IsamKeyStr; Ref : LongInt; KeyNr : Integer) : Boolean;
  begin
    DeleteACustomerKey := False;
    repeat
      BTDeleteKey(CustomerFB, KeyNr, Ref, K);
      if LockAbort then
        Exit;
    until not LockEnd;
    DeleteACustomerKey := IsamOK;
  end;

  function ChangeACustomerKey(OldK, NewK : IsamKeyStr; Ref : LongInt;
                              KeyNr               : Integer) : Boolean;
  begin
    if DeleteACustomerKey(OldK, Ref, KeyNr) then
      ChangeACustomerKey := AddACustomerKey(NewK, Ref, KeyNr)
    else
      ChangeACustomerKey := False;
  end;

  function AddCustomerRecAndKeys(var Key : IsamKeyStr; var Ref : LongInt) : Boolean;
  var
    OK                  : Boolean;
  begin
    AddCustomerRecAndKeys := False;
    repeat
      BTLockFileBlock(CustomerFB);
      if not IsamOK then
        if YesNo('Unable to lock file. Abort') then
          Exit;
    until IsamOK;
    OK := AddCustomerData(Customer, Ref);
    if OK then begin
      Key := BuildCustomerKey(Customer, 1);
      OK := AddACustomerKey(Key, Ref, 1);
    end;
    BTUnlockFileBlock(CustomerFB);
    AddCustomerRecAndKeys := IsamOK and OK;
  end;

  function ChangeCustomerRecAndKeys(var OldCustomer     : CustomerDef;
                                    Ref                 : LongInt) : Boolean;
  var
    CurrentCustomer     : CustomerDef;
    OK                  : Boolean;

    function CompStruc(var S1, S2; Len : Word) : Integer;
    var
      A1                  : array[1..$FFFF] of Byte absolute S1;
      A2                  : array[1..$FFFF] of Byte absolute S2;
      I                   : Word;
    begin
      for I := 1 to Len do
        if A1[I] < A2[I] then begin
          CompStruc := - 1;
          Exit;
        end
        else if A1[I] > A2[I] then begin
          CompStruc := 1;
          Exit;
        end;
      CompStruc := 0;
    end;

  begin
    ChangeCustomerRecAndKeys := True;
    if CompStruc(Customer, OldCustomer, SizeOf(CustomerDef)) = 0 then
      Exit;
    ChangeCustomerRecAndKeys := False;
    repeat
      BTLockFileBlock(CustomerFB);
      if not IsamOK then
        if YesNo('Unable to lock file. Abort') then
          Exit;
    until IsamOK;
    BTGetRec(CustomerFB, Ref, CurrentCustomer, False);
    if not IsamOK or (CurrentCustomer.Deleted <> 0) then begin
      BTUnlockFileBlock(CustomerFB);
      Exit;
    end;
    if CompStruc(CurrentCustomer, OldCustomer, SizeOf(CustomerDef)) <> 0 then begin
      BTUnlockFileBlock(CustomerFB);
      ShowMessage('The record has been changed in the meantime.', 'User Message');
      Exit;
    end;
    OK := ChangeCustomerData(Customer, Ref);
    if OK then
      if BuildCustomerKey(CurrentCustomer, 1) <> BuildCustomerKey(Customer, 1) then
        OK := ChangeACustomerKey(
          BuildCustomerKey(CurrentCustomer, 1), BuildCustomerKey(Customer, 1), Ref, 1);
    BTUnlockFileBlock(CustomerFB);
    ChangeCustomerRecAndKeys := IsamOK and OK;
  end;

  function DeleteCustomerRecAndKeys(Ref : LongInt) : Boolean;
  var
    OK                  : Boolean;
  begin
    DeleteCustomerRecAndKeys := False;
    repeat
      BTLockFileBlock(CustomerFB);
      if not IsamOK then
        if YesNo('Unable to lock file. Abort') then
          Exit;
    until IsamOK;
    OK := DeleteACustomerKey(BuildCustomerKey(Customer, 1), Ref, 1);
    if OK then
      OK := DeleteCustomerData(Ref);
    BTUnlockFileBlock(CustomerFB);
    DeleteCustomerRecAndKeys := IsamOK and OK;
  end;

  function SearchCustomerForKey(var Key : IsamKeyStr; var DatRef : LongInt) : Boolean;
  begin
    SearchCustomerForKey := False;
    repeat
      BTSearchKey(CustomerFB, 1, DatRef, Key);
      if LockAbort then
        Exit;
    until not LockEnd;
    SearchCustomerForKey := IsamOK;
  end;

  function BuildCustomerKey(var P; KeyNr : Word) : IsamKeyStr;

    function StrToKey(S : String) : String;
    begin
      if Length(S) <> 0 then
        AnsiUpperBuff(@S[1], Length(S));
      StrToKey := S;
    end;

  begin
    with CustomerDef(P).Data do
      case KeyNr of
        1 : BuildCustomerKey := Copy(StrToKey(StrPas(CustNum)), 1, 5);
      else BuildCustomerKey := '';
      end;
  end;


  {Order specific functions}

  function OpenOrderFile : Boolean;
  var
    OK, OK1             : Boolean;
  begin
    OpenOrderFile := False;
    OK := False;
    repeat
      BTOpenFileBlock(
        OrderFB, OrderFBName, False, False, False, True);
      OK := IsamOK;
      if not OK then begin
        OK1 := False;
        if IsamError = 10010 then begin
          if YesNo('Index defective. Rebuild') then
            OK1 := RebuildOrderFile;
        end
        else if IsamError = 9903 then begin
          if YesNo('Order data file does not exist. Create') then begin
            OK1 := CreateOrderFile;
            if not OK1 then
              Exit;
          end
          else
            Exit;
        end
        else
          Exit;
      end;
    until OK;
    OpenOrderFile := True;
  end;

  function CreateOrderFile : Boolean;
  var
    IID                 : IsamIndDescr;
  begin
    IID[1].Keyl := 5;
    IID[1].AllowDupK := True;
    BTCreateFileBlock(OrderFBName, SizeOf(OrderDef), 1, IID);
    CreateOrderFile := IsamOK;
  end;

  function RebuildOrderFile : Boolean;
  var
    IID                 : IsamIndDescr;
  begin
    IID[1].Keyl := 5;
    IID[1].AllowDupK := True;
    RebuildFileBlock(
      OrderFBName, SizeOf(OrderDef), 1, IID, @BuildOrderKey);
    RebuildOrderFile := IsamOK;
  end;

  function AddOrderData(P : OrderDef; var Ref : LongInt) : Boolean;
  begin
    AddOrderData := False;
    repeat
      BTAddRec(OrderFB, Ref, P);
      if LockAbort then
        Exit;
    until not LockEnd;
    AddOrderData := IsamOK;
  end;

  function ChangeOrderData(P : OrderDef; Ref : LongInt) : Boolean;
  begin
    ChangeOrderData := False;
    repeat
      BTPutRec(OrderFB, Ref, P, False);
      if LockAbort then
        Exit;
    until not LockEnd;
    ChangeOrderData := IsamOK;
  end;

  function DeleteOrderData(Ref : LongInt) : Boolean;
  begin
    DeleteOrderData := False;
    repeat
      BTDeleteRec(OrderFB, Ref);
      if LockAbort then
        Exit;
    until not LockEnd;
    DeleteOrderData := IsamOK;
  end;

  function AddAOrderKey(K : IsamKeyStr; Ref : LongInt; KeyNr : Integer) : Boolean;
  begin
    AddAOrderKey := False;
    repeat
      BTAddKey(OrderFB, KeyNr, Ref, K);
      if LockAbort then
        Exit;
    until not LockEnd;
    AddAOrderKey := IsamOK;
  end;

  function DeleteAOrderKey(K : IsamKeyStr; Ref : LongInt; KeyNr : Integer) : Boolean;
  begin
    DeleteAOrderKey := False;
    repeat
      BTDeleteKey(OrderFB, KeyNr, Ref, K);
      if LockAbort then
        Exit;
    until not LockEnd;
    DeleteAOrderKey := IsamOK;
  end;

  function ChangeAOrderKey(OldK, NewK : IsamKeyStr; Ref : LongInt;
                           KeyNr               : Integer) : Boolean;
  begin
    if DeleteAOrderKey(OldK, Ref, KeyNr) then
      ChangeAOrderKey := AddAOrderKey(NewK, Ref, KeyNr)
    else
      ChangeAOrderKey := False;
  end;

  function AddOrderRecAndKeys(var Key : IsamKeyStr; var Ref : LongInt) : Boolean;
  var
    OK                  : Boolean;
  begin
    AddOrderRecAndKeys := False;
    repeat
      BTLockFileBlock(OrderFB);
      if not IsamOK then
        if YesNo('Unable to lock file. Abort') then
          Exit;
    until IsamOK;
    OK := AddOrderData(Order, Ref);
    if OK then begin
      Key := BuildOrderKey(Order, 1);
      OK := AddAOrderKey(Key, Ref, 1);
    end;
    BTUnlockFileBlock(OrderFB);
    AddOrderRecAndKeys := IsamOK and OK;
  end;

  function ChangeOrderRecAndKeys(var OldOrder        : OrderDef;
                                 Ref                 : LongInt) : Boolean;
  var
    CurrentOrder        : OrderDef;
    OK                  : Boolean;

    function CompStruc(var S1, S2; Len : Word) : Integer;
    var
      A1                  : array[1..$FFFF] of Byte absolute S1;
      A2                  : array[1..$FFFF] of Byte absolute S2;
      I                   : Word;
    begin
      for I := 1 to Len do
        if A1[I] < A2[I] then begin
          CompStruc := - 1;
          Exit;
        end
        else if A1[I] > A2[I] then begin
          CompStruc := 1;
          Exit;
        end;
      CompStruc := 0;
    end;

  begin
    ChangeOrderRecAndKeys := True;
    if CompStruc(Order, OldOrder, SizeOf(OrderDef)) = 0 then
      Exit;
    ChangeOrderRecAndKeys := False;
    repeat
      BTLockFileBlock(OrderFB);
      if not IsamOK then
        if YesNo('Unable to lock file. Abort') then
          Exit;
    until IsamOK;
    BTGetRec(OrderFB, Ref, CurrentOrder, False);
    if not IsamOK or (CurrentOrder.Deleted <> 0) then begin
      BTUnlockFileBlock(OrderFB);
      Exit;
    end;
    if CompStruc(CurrentOrder, OldOrder, SizeOf(OrderDef)) <> 0 then begin
      BTUnlockFileBlock(OrderFB);
      ShowMessage('The record has been changed in the meantime.', 'User Message');
      Exit;
    end;
    OK := ChangeOrderData(Order, Ref);
    if OK then
      if BuildOrderKey(CurrentOrder, 1) <> BuildOrderKey(Order, 1) then
        OK := ChangeAOrderKey(
          BuildOrderKey(CurrentOrder, 1), BuildOrderKey(Order, 1), Ref, 1);
    BTUnlockFileBlock(OrderFB);
    ChangeOrderRecAndKeys := IsamOK and OK;
  end;

  function DeleteOrderRecAndKeys(Ref : LongInt) : Boolean;
  var
    OK                  : Boolean;
  begin
    DeleteOrderRecAndKeys := False;
    repeat
      BTLockFileBlock(OrderFB);
      if not IsamOK then
        if YesNo('Unable to lock file. Abort') then
          Exit;
    until IsamOK;
    OK := DeleteAOrderKey(BuildOrderKey(Order, 1), Ref, 1);
    if OK then
      OK := DeleteOrderData(Ref);
    BTUnlockFileBlock(OrderFB);
    DeleteOrderRecAndKeys := IsamOK and OK;
  end;

  function SearchOrderForKey(var Key : IsamKeyStr; var DatRef : LongInt) : Boolean;
  begin
    SearchOrderForKey := False;
    repeat
      BTSearchKey(OrderFB, 1, DatRef, Key);
      if LockAbort then
        Exit;
    until not LockEnd;
    SearchOrderForKey := IsamOK;
  end;

  function BuildOrderKey(var P; KeyNr : Word) : IsamKeyStr;

    function StrToKey(S : String) : String;
    begin
      if Length(S) <> 0 then
        AnsiUpperBuff(@S[1], Length(S));
      StrToKey := S;
    end;

  begin
    with OrderDef(P).Data do
      case KeyNr of
        1 : BuildOrderKey := Copy(StrToKey(StrPas(CustNum)), 1, 5);
      else BuildOrderKey := '';
      end;
  end;


end.

