
     {*******************************************************************}
     {*                          TWOBRWIN.PAS                           *}
     {*                    Version 1.00 January 1993                    *}
     {*              Copyright (c) 1993 TurboPower Software             *}
     {*                                                                 *}
     {*                         By Lee Inman,                           *}
     {*                      TurboPower Software                        *}
     {*******************************************************************}

{$I BRDEFOPT.INC}
{$X+}
{$M 16384,8192}

program TwoBrWin;
  {-Demonstration program for multiple browser objects and B-Tree Filer
  using a windows MDI application and OWL. The Customer file is related
  to the order file by the customer number.}

{$R TWOBRWIN.RES} {Link the resource file}

uses
  WinTypes,
  WinProcs,
{$IFDEF Ver70}
  Objects,
  OWindows,
  ODialogs,
{$ELSE}
  WObjects,
{$ENDIF}
  Strings,
  Filer,
  Rebuild,
  IsamTool,

  LowBrows,       {Low level windows file browser unit}
  WBrowser,       {Windows file browser unit}
  BWCC,           {Borland custom controls}

  BrWinCus,       {Customer window}
  BrWinOrd,       {Order window}
  BrWinDat,       {Types, constants, and global variables}
  BrWinUtl;       {Utilities and common file functions}

type
  PDemoApp            = ^TDemoApp;
  TDemoApp            = object(TApplication)
    constructor Init(AName : PChar);
    procedure InitMainWindow; virtual;
  end;

  PMainWin            = ^TMainWin;
  TMainWin            = object(TMDIWindow)
    CustomerBR        : PCustomerWin;
    OrderBR           : POrderWin;
    hCustomer         : THandle;

    constructor Init(ATitle : PChar; AMenu : hMenu);
    destructor Done;
      virtual;
    procedure SetupWindow;
      virtual;
    procedure GetWindowClass(var WndClass : TWndClass);
      virtual;

    procedure ForceUpdate;
    procedure CMTileChildren(var Msg: tMessage);
      virtual cm_First + cm_TileChildren;
    procedure CMCascadeChildren(var Msg: tMessage);
      virtual cm_First + cm_CascadeChildren;

    function InitBrowser : Boolean;
      {-Create and display the two browser windows}
    function InitDatabase : Boolean;
      {-Open the customer and order databases}

    procedure About(var Msg : TMessage);
      virtual cm_First + idm_About;
      {-Show the about dialog}
    procedure CustomerNew(var Msg : TMessage);
      virtual cm_First + idm_CNew;
      {-Allow entry of a new customer}
    procedure CustomerDelete(var Msg : TMessage);
      virtual cm_First + idm_CDelete;
      {-Allow the currently selected customer to be deleted}
    procedure CustomerSearch(var Msg : TMessage);
      virtual cm_First + idm_CSearch;
      {-Allow searching based on any field}
    procedure CustomerEdit(var Msg : TMessage);
      virtual cm_First + idm_CEdit;
      {-Allow editing of the current customer}
    procedure CustomerStatus(var Msg : TMessage);
      virtual cm_First + idm_CStatus;
      {-Display a dialog showing the customer db status}

    procedure OrderNew(var Msg : TMessage);
      virtual cm_First + idm_ONew;
      {-Allow entry of a new order}
    procedure OrderDelete(var Msg : TMessage);
      virtual cm_First + idm_ODelete;
      {-Allow the currently selected order to be deleted}
    procedure OrderEdit(var Msg : TMessage);
      virtual cm_First + idm_OEdit;
      {-Allow editing of the current order}
    procedure OrderFilterOnOff(var Msg : TMessage);
      virtual cm_First + idm_OFilterOnOff;
      {-Allow filtering of order records}
    procedure OrderStatus(var Msg : TMessage);
      virtual cm_First + idm_OStatus;
      {-Display a dialog showing the order db status}

    procedure CMDblClkCustomer(var Msg : TMessage);
      virtual cm_First + cm_DblClkCust;
      {-Double click invokes the CustomerEdit method}
    procedure CMDblClkOrder(var Msg : TMessage);
      virtual cm_First + cm_DblClkOrder;
      {-Double click invokes the OrderEdit method}
    procedure CMUpdateOrders(var Msg : TMessage);
      virtual cm_First + cm_UpdateOrders;
      {-Changes the order key range and updates the order window}
    procedure CMWindowTab(var Msg : TMessage);
      virtual cm_First + cm_WindowTab;
      {-Toggles focus from one window to the other}

    procedure GenerateRandomData;
      virtual cm_First + idm_RandomData;
      {-Creates customer and order records using random data}
  end;


  PNetDialog          = ^TNetDialog;
  TNetDialog          = object(TDialog)
    constructor Init(    AParent : PWindowsObject;
                         AName   : PChar;
                     var Buffer  : NetTypeRec);
  end;


  {TNetDialog}
  constructor TNetDialog.Init(    AParent : PWindowsObject;
                                  AName   : PChar;
                              var Buffer  : NetTypeRec);
  var
    P                   : PRadioButton;
  begin
    TDialog.Init(AParent, AName);
    TransferBuffer := @Buffer;
    New(P, InitResource(@Self, 101));
    New(P, InitResource(@Self, 102));
    New(P, InitResource(@Self, 103));
  end;


  {TMainWin}
  constructor TMainWin.Init(ATitle : PChar; AMenu : hMenu);
  begin
    TMDIWindow.Init(ATitle, AMenu);

    {Initialize Filer and open the fileblock}
    if not InitDatabase then begin
      DbError('Fileblock could not be initialized.');
      Halt;
    end;

    if not InitBrowser then
      Fail;
  end;

  procedure TMainWin.SetupWindow;
  begin
    TMDIWindow.SetupWindow;

    {Initilally tile the two browser windows}
    PostMessage(hWindow, wm_Command, cm_TileChildren, 0);

    {Store the handle of the customer browser window to be used to}
    {tab back and forth between customer and order windows}
    UpdateFocusChild;
    hCustomer := FocusChildHandle;
  end;

  destructor TMainWin.Done;
  begin
    TMDIWindow.Done;

    {-Order is this way because Isam or Shell are called indirectly}
    BTCloseFileBlock(CustomerFB);
    BTCloseFileBlock(OrderFB);
    BTExitIsam;
  end;

  procedure TMainWin.GetWindowClass(var WndClass : TWndClass);
  begin
    TMDIWindow.GetWindowClass(WndClass);
    {Get our icon for the demo}
    WndClass.hIcon := LoadIcon(hInstance, PChar(id_Icon));
  end;

  procedure tMainWin.ForceUpdate;
    {-Force painting of the bottom of each window}
  var
    R                : tRect;
    UsedDC           : hDC;
    TM               : tTextMetric;
    H                : Integer;

  begin
    {A small area below the last line displayed is not painted in some situations.}
    {i.e. if both windows are moved below the bottom of the main window, worked with}
    { and then tiled or moved so that the bottom of the window is visable. To see why}
    { this happens, look at the Paint and DisplayRow methods in WBrowser.}

    {Get info about current text size}
    UsedDC := GetDC(CustomerBR^.hWindow);
    if not GetTextMetrics(UsedDC,TM) then Exit;
    H := TM.tmHeight + TM.tmExternalLeading;
    ReleaseDC(CustomerBR^.hWindow,UsedDC);

    GetClientRect(CustomerBR^.hWindow,R);
    R.Top := R.Bottom - H;
    {Erase an area of one line hieght at the bottom of the Customer window}
    InvalidateRect(CustomerBr^.hWindow,@R,True);
    UpdateWindow(CustomerBR^.hWindow);

    GetClientRect(OrderBR^.hWindow,R);
    R.Top := R.Bottom - H;
    {Erase an area of one line hieght at the bottom of the Order window}
    InvalidateRect(OrderBr^.hWindow,@R,True);
    UpdateWindow(OrderBR^.hWindow);
  end;

  procedure tMainWin.CMTileChildren(var Msg: tMessage);
  begin
    tMdiWindow.CMTileChildren(Msg);
    {Insure that the complete window gets painted}
    ForceUpdate;
  end;

  procedure tMainWin.CMCascadeChildren(var Msg: tMessage);
  begin
    tMdiWindow.CMCascadeChildren(Msg);
    {Insure that the complete window gets painted}
    ForceUpdate;
  end;

  function TMainWin.InitDatabase : Boolean;
  var
    NTR                 : NetTypeRec;
    NetType             : NetSupportType;
  begin
    InitDatabase := False;

    NTR.NoNet := bf_Checked;
    NTR.MsNet := bf_Unchecked;
    NTR.Novell := bf_Unchecked;

    {If this is a network compile, query what type of net}
    if not BTNoNetCompiled then
      if Application^.ExecDialog(New(PNetDialog, Init(nil, 'NETTYPE', NTR))) = id_Cancel then
        Halt;

    if NTR.MsNet = bf_Checked then
      NetType := MsNet
    else if NTR.Novell = bf_Checked then
      NetType := Novell
    else
      NetType := NoNet;

    {Initialize filer leaving us all but 500K of heap space}
    BTInitIsam(NetType, MemAvail - 500000, 0);
    if IsamOK then begin
      InitDatabase := OpenCustomerFile;
      InitDatabase := OpenOrderFile;
    end else
      InitDatabase := False;
  end;

  function TMainWin.InitBrowser : Boolean;
  begin
    InitBrowser := False;

    {Create the browser windows}
    OrderBR := New(POrderWin,
      InitDemo(@Self, 'Order Information', OrderFB, 1, 50, 1, '', ''));
    if OrderBR = nil then
      Exit;

    CustomerBR := New(PCustomerWin,
      InitDemo(@Self, 'Customer Information', CustomerFB, 1, 50, 1, '', ''));
    if CustomerBR = nil then
      Exit;

    {Initialize and show the browser windows}
    OrderBR^.SetAndUpdateBrowserScreen('', 0);
    {Do customer window last so it has the focus}
    CustomerBR^.SetAndUpdateBrowserScreen('', 0);

    InitBrowser := True;
  end;

  procedure TMainWin.About(var Msg : TMessage);
  begin
    Application^.ExecDialog(New(PDialog, Init(@Self, 'ABOUT')));
  end;

  procedure TMainWin.CustomerNew(var Msg : TMessage);
  begin
    CustomerBR^.NewRec;
  end;

  procedure TMainWin.CustomerDelete(var Msg : TMessage);
  var
    Match : Boolean;
  begin
    {If a customer is selected}
    if CustomerBR^.GetCurrentRec(Match) > 0 then
      Exit;
    {No checking is done to see if this record has associated orders}
    CustomerBR^.DelRec
  end;

  procedure TMainWin.CustomerSearch(var Msg : TMessage);
  begin
    CustomerBR^.Search;
  end;

  procedure TMainWin.CustomerEdit(var Msg : TMessage);
  var
    Match : Boolean;
  begin
    {If a customer is selected}
    if CustomerBR^.GetCurrentRec(Match) > 0 then
      Exit;
    CustomerBR^.EditRec;
  end;

  procedure TMainWin.CustomerStatus(var Msg : TMessage);
  begin
    CustomerBR^.ShowStatus;
  end;

  procedure TMainWin.OrderNew(var Msg : TMessage);
  var
    Match : Boolean;
  begin
    {If a customer is selected}
    if CustomerBR^.GetCurrentRec(Match) > 0 then
      Exit;
    OrderBR^.NewRec(Customer.Data.CustNum);
  end;

  procedure TMainWin.OrderDelete(var Msg : TMessage);
  var
    Match : Boolean;
  begin
    {If an order is selected}
    if OrderBR^.GetCurrentRec(Match) > 0 then
      Exit;
    OrderBR^.DelRec;
  end;

  procedure TMainWin.OrderEdit(var Msg : TMessage);
  var
    Match : Boolean;
  begin
    {If an order is selected}
    if OrderBR^.GetCurrentRec(Match) > 0 then
      Exit;
    OrderBR^.EditRec;
  end;

  procedure TMainWin.OrderFilterOnOff(var Msg : TMessage);
  var
    M : hMenu;
  begin
    OrderBR^.FilterOnOff;
    {-Toggle the check mark next to this menu item}
    M := GetMenu(hWindow);
    if GetMenuState(M,idm_OFilterOnOff,mf_ByCommand) = bf_UnChecked then
      CheckMenuItem(M,idm_OFilterOnOff,mf_ByCommand or mf_Checked)
    else
      CheckMenuItem(M,idm_OFilterOnOff,mf_ByCommand or mf_UnChecked);
  end;

  procedure TMainWin.OrderStatus(var Msg : TMessage);
  begin
    OrderBR^.ShowStatus;
  end;

  procedure TMainWin.GenerateRandomData;
  var
    I, j                : Integer;
    Key                 : IsamKeyStr;
    Ref                 : LongInt;
    HoldNum             : array[0..20] of Char;

    function ZeroPadNum(N : Word) : String;
    var
      I                   : Word;
      Tmp                 : String;
    begin
      Tmp := '';
      for I := 1 to N do
        Tmp := Tmp + Chr(Random(10) + Ord('0'));
      ZeroPadNum := Tmp;
    end;

    function RanStr(N : Word) : String;
    var
      I                   : Word;
      Tmp                 : String;
    begin
      Tmp := '';
      for I := 0 to Random(N) do
        Tmp := Tmp + Chr(Random(26) + Ord('A'));
      RanStr := Tmp;
    end;

  begin
    SetCursor(LoadCursor(0, idc_Wait));
    Randomize;
    for I := 1 to NumCustomerGen do begin
      FillChar(Customer, SizeOf(Customer), #0);
      with Customer.Data do begin
        StrPCopy(CustNum, ZeroPadNum(5));
        StrCopy(HoldNum, CustNum);
        StrPCopy(LastName, RanStr(15));
        StrPCopy(FirstName, RanStr(15));
        StrPCopy(Address, RanStr(25));
        StrPCopy(City, RanStr(20));
        StrPCopy(State, RanStr(1)+RanStr(1));
        StrPCopy(Zip, ZeroPadNum(5));
        StrPCopy(Phone, ZeroPadNum(10));
      end;

      {The customer record won't be added if it happens to be a duplicate
       - but the orders will still be added}
      AddCustomerRecAndKeys(Key, Ref);

      FillChar(Order, SizeOf(Order), #0);
      for j := 0 to Random(NumOrderGen) do begin
        with Order.Data do begin
          StrPCopy(CustNum, HoldNum);
          StrPCopy(Product, RanStr(25));
          Str(Random(10000) * Random:8:2, Amount);
          Str(Random(10000) * Random:8:2, Balance);
        end;
        AddOrderRecAndKeys(Key, Ref);
      end;
    end;

    SetCursor(LoadCursor(0, idc_Arrow));

    CustomerBR^.UpdateBrowserScreen;

    {Uses literal values for this message not the global constants
     NumCustomerGen and NumOrderGen}
    MessageBox(hWindow,
       'Created 10 new customers (randomly)'#10#13+
       'each with 1 to 5 order entries.',
       'User Information', mb_OK);
  end;

  procedure TMainWin.CMDblClkCustomer(var Msg : TMessage);
  begin
    {Call the edit method}
    CustomerBR^.EditRec;
  end;

  procedure TMainWin.CMDblClkOrder(var Msg : TMessage);
  begin
    {Call the edit method}
    OrderBR^.EditRec;
  end;

  procedure TMainWin.CMUpdateOrders(var Msg : TMessage);
  begin
    with CurrentCustomer do begin
      {Change the key range used by the order browser}
      OrderBR^.ChangeKeyRange(StrPas(CustNum),StrPas(CustNum));
      {Inform the order browser to update the window with the new key range
       of records highlighting the first entry}
      OrderBR^.SetAndUpdateBrowserScreen(StrPas(CustNum),1);
    end;
  end;

  procedure TMainWin.CMWindowTab(var Msg : TMessage);
  var
    H                   : THandle;
  begin
    {Force FocusChildHandle to be updated}
    UpdateFocusChild;
    H := FocusChildHandle;

    {hCustomer is initialized in SetupWindow and should hold the handle
     to the customer window.}
    {Send message to the client window to activate the child window}
    if H = hCustomer then
      PostMessage(ClientWnd^.hWindow, wm_MDIActivate, Word(OrderBR^.hWindow), 0)
    else
      PostMessage(ClientWnd^.hWindow, wm_MDIActivate, Word(CustomerBR^.hWindow), 0);
  end;


  {TDemoApp}
  constructor TDemoApp.Init(AName : PChar);
  begin
    if not TApplication.Init(AName) then
      Fail;
  end;

  procedure TDemoApp.InitMainWindow;
  begin
    MainWindow := New(PMainWin,
      {The demo window title}
      Init('B-Tree Filer and Browser MDI Demo',
      {Get our menu from the resource}
      LoadMenu(hInstance, 'MAINMENU')));
  end;

var
  SaveExitProc        : Pointer;
  DemoApp             : TDemoApp;

  procedure DemoExitProc; Far;
  begin
    AutoErrorDisplay;
    ExitProc := SaveExitProc;
  end;

begin

  if DemoApp.Init('Browser Demo') then begin
    SaveExitProc := ExitProc;
    ExitProc := @DemoExitProc;
    DemoApp.Run;
    DemoApp.Done;
  end;

end.

